      PROGRAM DSAEXP
C*************************************************************
C*
C*     PROGRAMA PRINCIPAL
c*
c* programa para calcular errores locales en el caso DMPK,
c* utilizando un polinomio de grado k, en cada celda.(K=1,2)
C*
C*************************************************************
C*     NMA=NUMERO DE MATERIALES
C*     NGR=NUMERO DE GRUPOS
C*     NIX=NUMERO DE INTERVALOS EN X
C*     NAS=NUMERO DE ANISOTROPIA:
C*          0 => DISPERSION ISOTROPICA
C*         >0 => DISPERSION ANISOTROPICA
C*     NSN=NUMERO DE DIRECCIONES ANGULARES PARA LA APROXIMACION
C*         SN(2,4,8,16).
C*     IMP=NUMERO DE PUNTOS DE IMPRESION EN X
C*     NLBC=NUMERO DE CONDICION DE FRONTERA A LA IZQUIERDA:
C*          0 => CONDICION DE FLUJO CERO
C*          1 => CONDICION DE CORRIENTE CERO
C*     NRBC=NUMERO DE CONDICION DE FRONTERA A LA DERECHA:
C*          0 => CONDICION DE FLUJO CERO
C*          1 => CONDICION DE CORRIENTE CERO
C*     IPR=NUMERO DE PROBLEMA:
C*          1 => RESUELVE LA ECUACION DE TRANSPORTE CON FUENTE
C*               EXTERNA, PERO SIN FISIONES.
C*          2 => CALCULA EL NUMERO DE NEUTRONES SECUNDARIOS POR
C*               COLISION(FUENTE EXTERNA CERO).
C*          3 => CALCULA EL FACTOR DE MULTIPLICACION EFECTIVA
C*               (FUENTE EXTERNA CERO).
C*     NOPT=NUMERO DE OPCION:
C*          1 =>STEP DISCONTINUOUS SCHEME
C*          2 =>LINEAR DISCONTINUOUS SCHEME
C*          3 =>QUADRATIC DISCONTINUOUS SCHEME
C*     LMAX=NUMERO DE TERMINOS DE LA EXPANSION POR POLINOMIOS
C*          DE LEGENDRE DE LAS DISPERSIONES:
C*          1 =>DISPERSION ISOTROPICA
C*         >1 =>DISPERSION ANISOTROPICA
C*
C*
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION A(8000),NC(150)
       IFIMAX=8000
   10  READ(5,*)NMA,NGR,NIX,NAS,NSN,IMP,NLBC,NRBC,LMAX,IPR,NOPT
       IF(NMA.EQ.0) STOP
       WRITE(6,200)NMA,NGR,NIX,NAS,NSN,IMP,NLBC,NRBC,LMAX,IPR,NOPT
       CALL NLOGIC(N1,N2,N3,NOPT)
       RN=1.D0
       NIXA=IABS(NIX)
       NPX=NIXA+1
       NGR2=NGR*NGR
       IST=1
       ISS=IST+NGR*NMA
       ISF=ISS+NGR*NMA*NGR*LMAX
       IXP=ISF+NGR*NMA
       IX=IXP+NGR*NMA
       IQ=IX+NPX
       IFMU=IQ+NGR*NMA*LMAX
       IWMU=IFMU+NSN
       IFO=IWMU+NSN
       IFN=IFO+NPX*NGR*NSN
       IFT=IFN+NPX*NGR*NSN
       IFPO=IFT+NGR*NPX
       IFPN=IFPO+NGR*NSN*NIXA*N2
       IFPOX=IFPN+NGR*NSN*NIXA*N2
       IFPNX=IFPOX+NGR*NSN*NIXA*N3
       IS0=IFPNX+NGR*NSN*NIXA*N3
       IS1=IS0+NIXA*NGR*NSN
       IS2=IS1+NIXA*NGR*NSN*N2
       IXIMP=IS2+NIXA*NGR*NSN*N3
       IFIMP=IXIMP+IMP
       IVN=IFIMP+IMP*NGR*NSN
       IVPN=IVN+NPX*NGR*NSN
       IVPNX=IVPN+NIXA*NGR*NSN*N2
       IVT=IVPNX+NIXA*NGR*NSN*N3
       IF(IVT.GT.IFIMAX) STOP
       ITEX=0
       ITIN=0
C*
C*  LECTURA DE LOS PARAMETROS DE ENTRADA.
C*
       CALL INPUT(NMA,NIX,NIXA,NPX,NGR,NSN,NGR2,A(IST),A(ISS),A(ISF),
     1 A(IXP),NC,A(IFMU),A(IWMU),A(IX),A(IQ),EPSL,EPSR,LMAX)
C*
C*  CALCULO DE LOS VECTORES DE ENTRADA.
C*
       CALL INVAL(NPX,NGR,NSN,NIXA,NOPT,A(IFO),A(IFN),
     1 A(IFPO),A(IFPN),A(IFPOX),A(IFPNX),A(IVN),A(IVPN),
     2 A(IVPNX),NLBC,NRBC)
C*
   20  ITEX=ITEX+1
       RO=RN
       IF(IPR.EQ.1) GO TO 30
       ITIN=0
   30  CALL EQUAL(NPX,NGR,NSN,NIXA,NOPT,A(IVN),A(IFN),A(IVPN),
     1 A(IFPN),A(IVPNX),A(IFPNX))
   40  CONTINUE
C*
C*  CALCULO DE LOS TERMINOS DE FUENTE S0,S1,S2...,PARA CADA PUNTO,
C*  GRUPO Y DIRECCION ANGULAR.
C*
       CALL NUMIN(NMA,NIXA,NPX,NGR,NSN,NOPT,NGR2,A(IST),A(ISS),
     1 NC,A(IFMU),A(IWMU),A(IX),A(IQ),A(ISF),A(IXP),A(IFN),A(IS0),A(IS1)
     2 ,A(IS2),LMAX,ITEX,A(IFPN),A(IFPNX),A(IVN),A(IVPN),A(IVPNX))
   50  ITIN=ITIN+1
C*
C*  CALCULO DEL FLUJO DE NEUTRONES EN CADA PUNTO,GRUPO Y DIRECCION
C*  ANGULAR.
C*
       CALL AFLUX(NMA,NIXA,NPX,NGR,NSN,NOPT,A(IST),NC,A(IFMU),A(IX),
     1 A(IFN),A(IS0),A(IS1),A(IS2),NLBC,NRBC,A(IFPN),A(IFPNX))
C*
C*  CALCULO DEL ERROR ENTRE DOS APROXIMACIONES SUCESIVAS.
C*
       CALL ERROR(NPX,NGR,NSN,NIXA,NOPT,A(IFO),A(IFN),A(IFPO),A(IFPN)
     1 ,A(IFPOX),A(IFPNX),EMAX)
C*
C*  INICIALIZACION PARA LA PROXIMA ITERACION.
C*
       CALL EQUAL(NPX,NGR,NSN,NIXA,NOPT,A(IFO),A(IFN),A(IFPO),
     1 A(IFPN),A(IFPOX),A(IFPNX))
       IF(ITIN.GT.500) GO TO 60
       IF(EMAX.GT.EPSL.AND.IPR.EQ.3) GO TO 40
       IF(EMAX.GT.EPSL.AND.IPR.EQ.2) GO TO 50
       IF(EMAX.GT.EPSL.AND.IPR.EQ.1) GO TO 20
C*
C*  CALCULO DE LA RAPIDEZ DE REACCIONES.
C*
   60  RRFS=RR(NMA,NIXA,NPX,NGR,NSN,NOPT,IPR,A(IX),NC,A(IFMU),A(IFN),
     1 A(IS0),A(IS1),A(IS2),A(ISF),A(IWMU),NGR2,A(IFPN),A(IFPNX))
C*     IF(IPR.EQ.2) Q=1.D0/RN
       IF(IPR.EQ.3) Q=1.D0/RRFS
       IF(IPR.EQ.2) WRITE(6,300)RN,ITIN,ITEX
       IF(IPR.EQ.3) RN=RRFS
       IF(IPR.EQ.3) WRITE(6,400)RN,ITIN,ITEX
       IF(IPR.EQ.1) GO TO 70
       CALL MULTI(NIXA,NPX,NGR,NSN,NOPT,A(IFN),
     1 A(IFPN),A(IFPNX),Q)
       IF(DABS((RN-RO)/RN).GT.EPSR) GO TO 20
C*
  70   CALL INPOL(NMA,NIXA,NPX,NGR,NSN,IMP,RLM,NOPT,A(IX),
     1 NC,A(IFMU),A(IFN),A(IS0),A(IS1),A(IS2),A(IST),A(IXIMP),
     2 XMAX,XMIN,A(IFIMP),A(IFPN),A(IFPNX),A(IWMU),EN0,EN1,EN2,EN3)
C*
C*
C      IF(IMP.NE.101) GO TO 80
C*
C*  CALCULO DEL FLUJO DE NEUTRONES ESCALAR.
C*
       CALL TFLUX(IMP,NGR,NSN,A(IWMU),A(IFIMP),A(IFT))
C*
 80    CONTINUE
       GO TO 10
C*
C*  FORMATOS DE ENTRADA Y SALIDA.
C*
 100   FORMAT(2I2,I3,2I2,I3,5I2)
 200   FORMAT(/,3X,'NUMERO DE MATERIALES=',I2,10X,'NUMERO DE GRUPOS=',I2
     1 ,/,3X,'NUMERO DE INTERVALOS=',I3,10X,'NUMERO DE ANISOTROPIA=',I2
     2 ,/,3X,'NUMERO DE DIRECCIONES ANGULARES=',I2,/
     3 ,3X,'NUMERO DE PUNTOS DE IMPRESION=',I3,/
     4 ,3X,'NLBC=',I2,10X,'NRBC=',I2,10X,'LMAX=',I2,/
     5 ,3X,'NUMERO DE PROBLEMA=',I2,10X,'NUMERO DE OPCION=',I2,/)
 300   FORMAT(3X,'NUM. DE NEUT. SEC. POR COLISION=',D18.10
     1 ,3X,'ITIN=',I3,3X,'ITEX=',I3)
 400   FORMAT(3X,'FACTOR DE MULT. EFECT.=',D20.10,3X,'ITIN=',I3,3X,'ITEX
     1 =',I3)
       END
C***************************************************************
C*
       SUBROUTINE AFLUX(NMA,NIXA,NPX,NGR,NSN,NOPT,ST,NC,FMU,X,FN,S0,
     1 S1,S2,NLBC,NRBC,FPN,FPNX)
C*
C*****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION ST(NGR,NMA),NC(NIXA),FMU(NSN),X(NPX),FN(NPX,NGR,NSN),
     1           S0(NIXA,NGR,NSN),S1(NIXA,NGR,NSN),S2(NIXA,NGR,NSN),
     2           FPN(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN)
       DO 1000 K=1,NSN
       DO 1000 J=1,NGR
       DO 900 I=1,NIXA
       L=I
       IF(K.GT.NSN/2)L=NIXA+1-I
       NCO=NC(L)
       XL=X(L)
       XR=X(L+1)
       DX=XR-XL
       EI=ST(J,NCO)*DX/FMU(K)
       GO TO(100,700,800),NOPT
C*
C*STEP DISCONTINUOUS SCHEME (NODAL VERSION)
C*
 100   IF(K.GT.NSN/2)GOTO 150
       IF(EI.EQ.0.D0)GOTO 110
       FN(L+1,J,K)=(FN(L,J,K)+EI*S0(L,J,K)/ST(J,NCO))/(1.D0+EI)
       GOTO 900
 110   FN(L+1,J,K)=FN(L,J,K)+DX*S0(L,J,K)/FMU(K)
       GOTO 900
 150   IF(EI.EQ.0.D0) GOTO 160
       FN(L,J,K)=(FN(L+1,J,K)-EI*S0(L,J,K)/ST(J,NCO))/(1.D0-EI)
        GO TO 900
 160   FN(L,J,K)=FN(L+1,J,K)-DX*S0(L,J,K)/FMU(K)
       GOTO 900
C*
C*LINEAR DISCONTINUOUS SCHEME (NODAL VERSION).
C*
 700   IF(K.GT.NSN/2)GOTO 750
       IF(EI.EQ.0.D0)GOTO 710
       FN(L+1,J,K)=((6.D0-2.D0*EI)*FN(L,J,K)+(EI/ST(J,NCO))*(S0(L,J,K)
     1 *(6.D0+EI)+EI*S1(L,J,K)))/(6.D0+4.D0*EI+EI*EI)
       FPN(L,J,K)=EI*(3.D0+EI)*S0(L,J,K)/ST(J,NCO)+(6.D0+EI)*FN(L,J,K)-
     1 S1(L,J,K)*EI/ST(J,NCO)
       FPN(L,J,K)=FPN(L,J,K)/(6.D0+4.D0*EI+EI*EI)
       GOTO 900
 710   FN(L+1,J,K)=FN(L,J,K)+DX*S0(L,J,K)/FMU(K)
       GOTO 900
 750   IF(EI.EQ.0.D0)GOTO 760
       FN(L,J,K)=((6.D0+2.D0*EI)*FN(L+1,J,K)-(EI/ST(J,NCO))*(S0(L,J,K)
     1 *(6.D0-EI)+EI*S1(L,J,K)))/(6.D0-4.D0*EI+EI*EI)
       FPN(L,J,K)=-EI*(3.D0-EI)*S0(L,J,K)/ST(J,NCO)+(6.D0-EI)*FN(L+1,J,K
     1 )-S1(L,J,K)*EI/ST(J,NCO)
       FPN(L,J,K)=FPN(L,J,K)/(6.D0-4.D0*EI+EI*EI)
       GO TO 900
 760   FN(L,J,K)=FN(L+1,J,K)-DX*S0(L,J,K)/FMU(K)
       GOTO 900
C*
C*QUADRATIC DISCONTINUOUS SCHEME (NODAL VERSION).
C*
 800   IF(K.GT.NSN/2)GOTO 850
       IF(EI.EQ.0.D0)GOTO 810
       QD=60.D0+36.D0*EI+9.D0*EI*EI+EI*EI*EI
       FN(L+1,J,K)=(60.D0-24.D0*EI+3.D0*EI*EI)*FN(L,J,K)+
     1 EI*((60.D0+6.D0*EI+EI*EI)*S0(L,J,K)+EI*(10.D0+EI)*S1(L,J,K)
     2 +EI*EI*S2(L,J,K))/ST(J,NCO)
       FN(L+1,J,K)=FN(L+1,J,K)/QD
       FPN(L,J,K)=(60.D0+6.D0*EI+EI*EI)*FN(L,J,K)+
     1 EI*((30.D0+8.D0*EI+EI*EI)*S0(L,J,K)-(10.D0+EI)*S1(L,J,K)
     2 -EI*S2(L,J,K))/ST(J,NCO)
       FPN(L,J,K)=FPN(L,J,K)/QD
       FPNX(L,J,K)=(-10.D0*EI-EI*EI)*FN(L,J,K)+EI*((10.D0+EI)
     1 *S0(L,J,K)+EI*(6.D0+EI)*S1(L,J,K)/3.D0-(2.D0+EI)*S2(L,J,K)
     2 )/ST(J,NCO)
       FPNX(L,J,K)=FPNX(L,J,K)/QD
       GO TO 900
 810   FN(L+1,J,K)=FN(L,J,K)+DX*S0(L,J,K)/FMU(K)
       FPN(L,J,K)=FN(L,J,K)+DX*(3.D0*S0(L,J,K)-S1(L,J,K))/(6.D0*FMU(K))
       FPNX(L,J,K)=DX*(5.D0*S0(L,J,K)-S2(L,J,K))/(30.D0*FMU(K))
       GO TO 900
 850   IF(EI.EQ.0.D0)GOTO 860
       QD=60.D0-36.D0*EI+9.D0*EI*EI-EI*EI*EI
       FN(L,J,K)=(60.D0+24.D0*EI+3.D0*EI*EI)*FN(L+1,J,K)
     1 -EI*((60.D0-6.D0*EI+EI*EI)*S0(L,J,K)+EI*(10.D0-EI)*S1(L,J,K)
     2 +EI*EI*S2(L,J,K))/ST(J,NCO)
       FN(L,J,K)=FN(L,J,K)/QD
       FPN(L,J,K)=(60.D0-6.D0*EI+EI*EI)*FN(L+1,J,K)
     1 -EI*((30.D0-8.D0*EI+EI*EI)*S0(L,J,K)+(10.D0-EI)*S1(L,J,K)
     2 +EI*S2(L,J,K))/ST(J,NCO)
       FPN(L,J,K)=FPN(L,J,K)/QD
       FPNX(L,J,K)=-(10.D0*EI-EI*EI)*FN(L+1,J,K)-EI*(-(10.D0-EI)
     1 *S0(L,J,K)-EI*(6.D0-EI)*S1(L,J,K)/3.D0+(2.D0-EI)*S2(L,J,K)
     2 )/ST(J,NCO)
       FPNX(L,J,K)=FPNX(L,J,K)/QD
       GO TO 900
 860   FN(L,J,K)=FN(L+1,J,K)-DX*S0(L,J,K)/FMU(K)
       FPN(L,J,K)=FN(L+1,J,K)+DX*(3.D0*S0(L,J,K)+S1(L,J,K))/
     1 (6.D0*FMU(K))
       FPNX(L,J,K)=DX*(5.D0*S0(L,J,K)-S2(L,J,K))/(30.D0*FMU(K))
       GOTO 900
 900   CONTINUE
       M=K
       N=NSN-K+1
       IF(NLBC.EQ.1.AND.K.GT.NSN/2)GO TO 910
       GO TO 930
 910   FN(1,J,N)=FN(1,J,M)
 930   IF(NRBC.EQ.1.AND.K.LE.NSN/2)GO TO 940
       GO TO 960
 940   FN(NPX,J,N)=FN(NPX,J,M)
 960   CONTINUE
 1000  CONTINUE
       RETURN
       END
C*************************************************************
C*
       SUBROUTINE INPUT(NMA,NIX,NIXA,NPX,NGR,NSN,NGR2,ST,SS,SF,XP,NC,
     1 FMU,WMU,X,Q,EPSL,EPSR,LMAX)
C*
C*************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION IC(3),ST(NGR,NMA),SS(NGR2,NMA,LMAX),SF(NGR,NMA),XP(NGR,
     1 NMA),NC(NIXA),X(NPX),Q(NGR,NMA,LMAX),FMU(NSN),WMU(NSN)
       DO 10 I=1,NMA
       READ(5,*)(ST(J,I),J=1,NGR)
 10    CONTINUE
       DO 20 L=1,LMAX
       DO 20 I=1,NMA
       KJ=(K-1)*NGR+J
       READ(5,*)(SS(KJ,I,L),KJ=1,NGR2)
 20    CONTINUE
       DO 25 I=1,NMA
       READ (5,*)(SF(J,I),J=1,NGR)
 25    CONTINUE
       DO 28 I=1,NMA
       READ(5,*)(XP(J,I),J=1,NGR)
 28    CONTINUE
 30    READ(5,*)IC(1),IC(2),IC(3)
       IF(IC(1).EQ.0)GO TO 45
       ICM1=IC(3)-1
       DO 40 M=IC(2),ICM1
       NC(M)=IC(1)
 40    CONTINUE
       GO TO 30
 45    READ(5,*) XI,XF
       WRITE(6,1350)XI,XF
       DO 50 I=1,NPX
       X(I)=XI+(I-1)*(XF-XI)/NIXA
 50    CONTINUE
       READ(5,*) EPSL,EPSR
       WRITE(6,1450)EPSL,EPSR
       DO 60 L=1,LMAX
       DO 60 I=1,NMA
       READ(5,*)(Q(J,I,L),J=1,NGR)
 60    CONTINUE
       CALL SNDATA(NSN,FMU,WMU)
C* 1050  FORMAT(3X,'ST(',I2,',',I2,')=',D8.2)
C* 1150  FORMAT(3X,'SS(',I2,',',I2,',',I2,',',I2,')=',D8.2)
C* 1250  FORMAT(3X,I3,',',I3,',',I3)
 1350  FORMAT(3X,'XI=',D10.4,3X,'XF=',D10.4)
 1450  FORMAT(3X,'ERROR MAXIMO=',D10.4,5X,'ERROR..=',D10.4)
C* 1550  FORMAT(3X,'Q(',I2,',',I2,',',I2,')=',D8.2)
C* 1850  FORMAT(3X,'SF(',I2,',',I2,')=',D8.2)
C* 1950  FORMAT(3X,'XP(',I2,',',I2,')=',D8.2)
       RETURN
       END
C************************************************************
C*
       SUBROUTINE INVAL(NPX,NGR,NSN,NIXA,NOPT,FO,FN,
     1 FPO,FPN,FPOX,FPNX,VN,VPN,VPNX,NLBC,NRBC)
C*
C***************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FO(NPX,NGR,NSN),FN(NPX,NGR,NSN),FPO(NIXA,NGR,NSN)
     1          ,FPN(NIXA,NGR,NSN),FPOX(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN)
     2          ,VN(NPX,NGR,NSN),VPN(NIXA,NGR,NSN),VPNX(NIXA,NGR,NSN)
       DO 10 K=1,NSN
       DO 10 J=1,NGR
       DO 10 I=1,NPX
       FO(I,J,K)=1.D0
       VN(I,J,K)=1.D0
 10    CONTINUE
       IF(NOPT.LT.2)GO TO 55
       DO 50 K=1,NSN
       DO 50 J=1,NGR
       DO 50 I=1,NIXA
       FPO(I,J,K)=1.D0
       VPN(I,J,K)=1.D0
  50   CONTINUE
  55   IF(NOPT.LT.3)GO TO 65
       DO 60 K=1,NSN
       DO 60 J=1,NGR
       DO 60 I=1,NIXA
       FPOX(I,J,K)=0.D0
       VPNX(I,J,K)=0.D0
  60   CONTINUE
  65   L=NSN/2
       DO 70 K=1,L
       M=NSN+1-K
       DO 70 J=1,NGR
       IF(NLBC.EQ.0) FO(1,J,K)=0.D0
       IF(NRBC.EQ.0)FO(NPX,J,M)=0.D0
 70    CONTINUE
       CALL EQUAL(NPX,NGR,NSN,NIXA,NOPT,FN,FO,FPN,FPO,FPNX,FPOX)
       RETURN
       END
C****************************************************************
C*
       SUBROUTINE MINMAX(N,F,FMIN,FMAX)
C*
C*************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION F(N)
       FMIN=F(1)
       FMAX=F(1)
       DO 20 I=2,N
       IF(FMIN.LT.F(I))GO TO 10
       FMIN=F(I)
  10   IF(FMAX.GT.F(I))GO TO 20
       FMAX=F(I)
  20   CONTINUE
       RETURN
       END
C****************************************************************
C*
       SUBROUTINE MULTI(NIXA,NPX,NGR,NSN,NOPT,FN,FPN,FPNX,R)
C*
C*****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FN(NPX,NGR,NSN),FPN(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN)
       DO 10 I=1,NPX
       DO 10 J=1,NGR
       DO 10 K=1,NSN
 10    FN(I,J,K)=FN(I,J,K)*R
       IF(NOPT.LT.2)RETURN
       DO 50 I=1,NIXA
       DO 50 J=1,NGR
       DO 50 K=1,NSN
  50   FPN(I,J,K)=FPN(I,J,K)*R
       IF(NOPT.LT.3)RETURN
       DO 60 I=1,NIXA
       DO 60 J=1,NGR
       DO 60 K=1,NSN
  60   FPNX(I,J,K)=FPNX(I,J,K)*R
       RETURN
       END
C*******************************************************************
C*
       SUBROUTINE NUMIN(NMA,NIXA,NPX,NGR,NSN,NOPT,NGR2,ST,SS,NC,
     1 FMU,WMU,X,Q,SF,XP,FN,S0,S1,S2,LMAX,ITEX,FPN,FPNX,VN,VPN,VPNX)
C*
C*****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION ST(NGR,NMA),SS(NGR2,NMA,LMAX),NC(NIXA),
     1           FMU(NSN),WMU(NSN),X(NPX),Q(NGR,NMA,LMAX),
     2           FN(NPX,NGR,NSN),S0(NIXA,NGR,NSN),S1(NIXA,NGR,
     3           NSN),S2(NIXA,NGR,NSN),SF(NGR,NMA),XP(NGR,NMA),
     4           FPN(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN),VN(NPX,NGR,NSN),
     5           VPN(NIXA,NGR,NSN),VPNX(NIXA,NGR,NSN)
      DO 10 IP=1,NIXA
      DO 10 IG=1,NGR
      DO 10 ID=1,NSN
      S0(IP,IG,ID)=0.0D0
      IF(NOPT.EQ.1)GO TO 10
      S1(IP,IG,ID)=0.0D0
      IF(NOPT.EQ.2)GO TO 10
      S2(IP,IG,ID)=0.0D0
   10 CONTINUE
      IF(NOPT.GT.1)GO TO 30
      DO 20 IP=1,NIXA
      NCO=NC(IP)
      DX=X(IP+1)-X(IP)
      DO 20 IG=1,NGR
      DO 20 ID=1,NSN
      U=FMU(ID)
      DO 20 L=1,LMAX
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)
      DO 20 N=1,NGR
      M=NSN/2
      DO 20 K=1,M
      NJ=(IG-1)*NGR+N
      W=WMU(K)
      T=FMU(K)
      EI=ST(N,NCO)*DX/T
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FN(IP+1,N,K)
      T=FMU(NSN+1-K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FN(IP,N,NSN+1-K)
      IF(L.GT.1)GO TO 20
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W
     1            *VN(IP+1,N,K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W
     1            *VN(IP,N,NSN+1-K)
   20 CONTINUE
      RETURN
   30 IF(NOPT.GT.2)GO TO 50
      DO 40 IP=1,NIXA
      DX=X(IP+1)-X(IP)
      NCO=NC(IP)
      DO 40 IG=1,NGR
      DO 40 ID=1,NSN
      U=FMU(ID)
      DO 40 L=1,LMAX
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)*0.0D0
      DO 40 N=1,NGR
      M=NSN/2
      DO 40 K=1,M
      NJ=(IG-1)*NGR+N
      W=WMU(K)
      T=FMU(K)
      EI=ST(N,NCO)*DX/T
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPN(IP,N,K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *(-FPN(IP,N,K)+FN(IP+1,N,K))
      T=FMU(NSN+1-K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPN(IP,N,NSN+1-K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *(FPN(IP,N,NSN+1-K)-FN(IP,N,NSN+1-K))
      IF(L.GT.1)GO TO 40
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPN(IP,N,K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPN(IP,N,NSN+1-K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*(-VPN(IP,N,K)
     1            +VN(IP+1,N,K))
      S1(IP,IG,ID)=S1(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*(VPN(IP,N,
     1             NSN+1-K)-VN(IP,N,NSN+1-K))
   40 CONTINUE
      RETURN
   50 IF(NOPT.GT.3)GO TO 70
      DO 60 IP=1,NIXA
      DX=X(IP+1)-X(IP)
      NCO=NC(IP)
      DO 60 IG=1,NGR
      DO 60 ID=1,NSN
      U=FMU(ID)
      DO 60 L=1,LMAX
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)*0.0D0
      S2(IP,IG,ID)=S2(IP,IG,ID)+(2*L-1)*P(L,U)*Q(IG,NCO,L)*0.0D0
      DO 60 N=1,NGR
      M=NSN/2
      DO 60 K=1,M
      NJ=(IG-1)*NGR+N
      W=WMU(K)
      T=FMU(K)
      EI=ST(N,NCO)*DX/T
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPN(IP,N,K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPNX(IP,N,K)*3.D0
      S2(IP,IG,ID)=S2(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *(FN(IP+1,N,K)-FPN(IP,N,K)-FPNX(IP,N,K)*3.D0)
      T=FMU(NSN+1-K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPN(IP,N,NSN+1-K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *FPNX(IP,N,NSN+1-K)*3.D0
      S2(IP,IG,ID)=S2(IP,IG,ID)+(2*L-1)*P(L,U)*SS(NJ,NCO,L)*W*P(L,T)
     1            *(FN(IP,N,NSN+1-K)-FPN(IP,N,
     2            NSN+1-K)+FPNX(IP,N,NSN+1-K)*3.D0)
      IF(L.GT.1)GO TO 60
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPN(IP,N,K)
      S0(IP,IG,ID)=S0(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPN(IP,N,NSN+1-K)
      S1(IP,IG,ID)=S1(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPNX(IP,N,K)*3.D0
      S1(IP,IG,ID)=S1(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*VPNX(IP,N,
     1             NSN+1-K)*3.D0
      S2(IP,IG,ID)=S2(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*(
     1             VN(IP+1,N,K)-VPN(IP,N,K)-VPNX(IP,N,K)*3.D0)
      S2(IP,IG,ID)=S2(IP,IG,ID)+XP(IG,NCO)*SF(N,NCO)*W*(
     1             VN(IP,N,NSN+1-K)-VPN(IP,N,NSN+1-K)
     2            +VPNX(IP,N,NSN+1-K)*3.D0)
   60 CONTINUE
      RETURN
   70 WRITE(6,100)NOPT
      RETURN
  100 FORMAT(/,10X,'ESTE CASO NO EXISTE NOPT=',3X,I3,/)
      END
****************************************************************
C*
       SUBROUTINE NLOGIC(N1,N2,N3,NOPT)
C*
C****************************************************************
       N1=1
       N2=1
       IF(NOPT.LT.2)N2=0
       N3=1
       IF(NOPT.NE.3)N3=0
       RETURN
       END
C*******************************************************************
C*
       FUNCTION RR(NMA,NIXA,NPX,NGR,NSN,NOPT,IPR,X,NC,FMU,FN,
     1 S0,S1,S2,SF,WMU,NGR2,FPN,FPNX)
C*
C******************************************************************
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       DIMENSION FMU(NSN),WMU(NSN),FN(NPX,NGR,NSN),S0(NIXA,NGR,NSN),
     1           S1(NIXA,NGR,NSN),S2(NIXA,NGR,NSN),NC(NIXA),X(NPX),
     2          SF(NGR,NMA),FPN(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN)
       IF(IPR.EQ.1)RETURN
       RR=0.0D0
       DO 10 I=1,NIXA
       NCO=NC(I)
       DX=X(I+1)-X(I)
       DO 10 N=1,NGR
       DO 10 K=1,NSN
       IF(NOPT.EQ.1.AND.FMU(K).GT.0.0)FCN=FN(I+1,N,K)
       IF(NOPT.EQ.1.AND.FMU(K).LT.0.0)FCN=FN(I,N,K)
       IF(NOPT.NE.1)FCN=FPN(I,N,K)
       RR=RR+WMU(K)*FCN*DX*SF(N,NCO)
   10  CONTINUE
C*
C*
       RETURN
       END
C*****************************************************************
C*
       SUBROUTINE SNDATA(NSN,FMU,WMU)
C*
C*****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FMU(NSN),WMU(NSN)
       IF(NSN.EQ.2)M=1
       IF(NSN.EQ.4)M=2
       IF(NSN.EQ.8)M=3
       IF(NSN.EQ.16)M=4
       GO TO(100,200,300,400),M
 100   FMU(1)=1.D0/DSQRT(3.D0)
       FMU(2)=-1.D0/DSQRT(3.D0)
       WMU(1)= 0.5D0
       WMU(2)=0.5D0
       RETURN
 200   FMU(1)=.861136311594053D0
       FMU(2)=.339981043584856D0
       FMU(3)=-.339981043584856D0
       FMU(4)=-.861136311594053D0
       WMU(1)=0.347854845137454D0*.5D0
       WMU(2)=0.652145154862546D0*.5D0
       WMU(3)=0.652145154862546D0*.5D0
       WMU(4)=0.347854845137454D0*.5D0
       RETURN
 300   FMU(1)=.960289856497536D0
       FMU(2)=.796666477413627D0
       FMU(3)=.525532409916329D0
       FMU(4)=.1834346424956507D0
       FMU(5)=-.183434642495650D0
       FMU(6)=-.525532409916329D0
       FMU(7)=-.796666477413627D0
       FMU(8)=-.960289856497536D0
       WMU(1)=.101228536290376D0*.5D0
       WMU(2)=.222381034453374D0*.5D0
       WMU(3)=.313706645877887D0*.5D0
       WMU(4)=.362683783378362D0*.5D0
       WMU(5)=.362683783378362D0*.5D0
       WMU(6)=.313706645877887D0*.5D0
       WMU(7)=.222381034453374D0*.5D0
       WMU(8)=.101228536290376D0*.5D0
       RETURN
 400   FMU(1)=.989400934991649D0
       FMU(2)=.944575023073232D0
       FMU(3)=.865631202387831D0
       FMU(4)=.755404408355003D0
       FMU(5)=.617876244402643D0
       FMU(6)=.458016777657227D0
       FMU(7)=.281603550779258D0
       FMU(8)=.095012509837637D0
       FMU(9)=-FMU(8)
       FMU(10)=-FMU(7)
       FMU(11)=-FMU(6)
       FMU(12)=-FMU(5)
       FMU(13)=-FMU(4)
       FMU(14)=-FMU(3)
       FMU(15)=-FMU(2)
       FMU(16)=-FMU(1)
       WMU(1)=.027152459411754D0*.5D0
       WMU(2)=.062253523938647D0*.5D0
       WMU(3)=.095158511682492D0*.5D0
       WMU(4)=.124628971255533D0*.5D0
       WMU(5)=.149595988816576D0*.5D0
       WMU(6)=.169156519395002D0*.5D0
       WMU(7)=.182603415044923D0*.5D0
       WMU(8)=.189450610455068D0*.5D0
       WMU(9)=WMU(8)
       WMU(10)=WMU(7)
       WMU(11)=WMU(6)
       WMU(12)=WMU(5)
       WMU(13)=WMU(4)
       WMU(14)=WMU(3)
       WMU(15)=WMU(2)
       WMU(16)=WMU(1)
       RETURN
       END
C*****************************************************************
C*
       SUBROUTINE TFLUX(IMP,NGR,NSN,WMU,FIMP,FT)
C*
C****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION WMU(NSN),FIMP(IMP,NGR,NSN),FT(IMP,NGR)
       DO 10 I=1,IMP
       DO 10 J=1,NGR
 10    FT(I,J)=0.D0
       DO 20 I=1,IMP
       DO 20 J=1,NGR
       DO 30 K=1,NSN
       FT(I,J)=FT(I,J)+WMU(K)*FIMP(I,J,K)
  30   CONTINUE
  20   CONTINUE
       RETURN
       END
C***************************************************************
C*
       FUNCTION FPQ(NMA,NIXA,NPX,NGR,NSN,RLM,NOPT,X,NC,FMU,FN,S0,
     1 S1,S2,ST,I,J,K,XC,FPN,FPNX)
C*
C*************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FMU(NSN),FN(NPX,NGR,NSN),S0(NIXA,NGR,NSN),
     1           S1(NIXA,NGR,NSN),S2(NIXA,NGR,NSN),NC(NIXA),X(NPX),
     2           ST(NGR,NMA),FPN(NIXA,NGR,NSN),FPNX(NIXA,NGR,NSN)
       NCO=NC(I)
       XL=X(I)
       XR=X(I+1)
       DX=XR-XL
       Z=(2.D0*XC-XL-XR)/DX
       GO TO(100,200,300),NOPT
C*
C*STEP DISCONTINUOUS SCHEME (NODAL VERSION)
C*
  100  IF(FMU(K).LT.0.0)GO TO 150
       U0=1.D0
       FPQ=FN(I+1,J,K)*U0
       RETURN
  150  U0=1.D0
       FPQ=FN(I,J,K)*U0
       RETURN
C*
C*LINEAR DISCONTINUOUS SCHEME (NODAL VERSION)
C*
 200   IF(FMU(K).LT.0.0)GO TO 250
       U0=Z
       V0=1.D0-Z
       FPQ=FN(I+1,J,K)*U0+FPN(I,J,K)*V0
       RETURN
 250   U0=-Z
       V0=1.D0+Z
       FPQ=FN(I,J,K)*U0+FPN(I,J,K)*V0
       RETURN
C*
C*QUADRATIC DISCONTINUOUS SCHEME (NODAL VERSION)
C*
 300   IF(FMU(K).LT.0.0)GO TO 350
       U0=(3.D0*Z*Z-1.D0)*0.5D0
       V0=3.D0*(1.0D0-Z*Z)*0.5D0
       V1=(1.0D0+3.D0*Z)*(1.D0-Z)*0.5D0
       FPQ=FN(I+1,J,K)*U0+FPN(I,J,K)*V0+FPNX(I,J,K)*V1*3.D0
       RETURN
  350  U0=(3.D0*Z*Z-1.D0)*0.5D0
       V0=3.D0*(1.D0-Z*Z)*0.5D0
       V1=-(1.D0-3.D0*Z)*(1.D0+Z)*0.5D0
       FPQ=FN(I,J,K)*U0+FPN(I,J,K)*V0+FPNX(I,J,K)*V1*3.D0
       RETURN
       END
C***************************************************************
C*
       FUNCTION P(N,Z)
C*
C****************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       GO TO(10,20,30,40,50),N
 10    P=1.D0
       RETURN
 20    P=Z
       RETURN
 30    P=0.5D0*(3.D0*Z*Z-1.D0)
       RETURN
 40    P=.5D0*Z*(5.D0*Z*Z-3.D0)
       RETURN
 50    Z2=Z*Z
       P=.125D0*(35.D0*Z2*Z2-30.D0*Z2+3.D0)
       RETURN
       END
C************************************************************
C*
       SUBROUTINE EQUAL(NPX,NGR,NSN,NIXA,NOPT,FO,FN,FPO,FPN,FPOX,FPNX)
C*
C*************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FO(NPX,NGR,NSN),FN(NPX,NGR,NSN),FPO(NIXA,NGR,NSN),
     1           FPN(NIXA,NGR,NSN),FPOX(NIXA,NGR,NSN),FPNX(NIXA,NGR
     2           ,NSN)
       DO 10 K=1,NSN
       DO 10 J=1,NGR
       DO 10 I=1,NPX
C*       FN(I,J,K)=FN(I,J,K)+W*(FN(I,J,K)-FO(I,J,K))
       FO(I,J,K)=FN(I,J,K)
 10    CONTINUE
       IF(NOPT.LT.2)RETURN
       DO 50 K=1,NSN
       DO 50 J=1,NGR
       DO 50 I=1,NIXA
C*       FPN(I,J,K)=FPN(I,J,K)+W*(FPN(I,J,K)-FPO(I,J,K))
       FPO(I,J,K)=FPN(I,J,K)
  50   CONTINUE
       IF(NOPT.LT.3)RETURN
       DO 60 K=1,NSN
       DO 60 J=1,NGR
       DO 60 I=1,NIXA
C*       FPNX(I,J,K)=FPNX(I,J,K)+W*(FPNX(I,J,K)-FPOX(I,J,K))
       FPOX(I,J,K)=FPNX(I,J,K)
  60   CONTINUE
       RETURN
       END
C************************************************************
C*
       SUBROUTINE ERROR(NPX,NGR,NSN,NIXA,NOPT,FO,FN,FPO,FPN,FPOX,FPNX,
     1 EMAX)
C*
C************************************************************
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)
       DIMENSION FO(NPX,NGR,NSN),FN(NPX,NGR,NSN),FPO(NIXA,NGR,NSN),
     1           FPN(NIXA,NGR,NSN),FPOX(NIXA,NGR,NSN),FPNX(NIXA,NGR
     2           ,NSN)
       EFMAX=DABS(FN(1,1,1)-FO(1,1,1))
       DO 10 K=1,NSN
       DO 10 J=1,NGR
       DO 10 I=1,NPX
       EFIJK=DABS(FN(I,J,K)-FO(I,J,K))
       IF(EFMAX.LT.EFIJK)EFMAX=EFIJK
 10    CONTINUE
       EMAX=EFMAX
       IF(NOPT.LT.2)RETURN
       EPMAX=DABS(FPN(1,1,1)-FPO(1,1,1))
       DO 50 K=1,NSN
       DO 50 J=1,NGR
       DO 50 I=1,NIXA
       EPIJK=DABS(FPN(I,J,K)-FPO(I,J,K))
       IF(EPMAX.LT.EPIJK)EPMAX=EPIJK
 50    CONTINUE
       IF(EMAX.LT.EPMAX)EMAX=EPMAX
       IF(NOPT.LT.3)RETURN
       EPXMAX=DABS(FPNX(1,1,1)-FPOX(1,1,1))
       DO 60 K=1,NSN
       DO 60 J=1,NGR
       DO 60 I=1,NIXA
       EPXIJK=DABS(FPNX(I,J,K)-FPOX(I,J,K))
       IF(EPXMAX.LT.EPXIJK)EPXMAX=EPXIJK
  60   CONTINUE
       IF(EMAX.LT.EPXMAX)EMAX=EPXMAX
       RETURN
       END
C***************************************************************        000010
C*                                                                      000020
       SUBROUTINE INPOL(NMA,NIXA,NPX,NGR,NSN,IMP,RLM,NOPT               000030
     1 ,X,NC,FMU,FN,S0,S1,S2,ST,XIMP,XMAX,XMIN,FIMP,FPN,FPNX,           000040
     2 WMU,EN0,EN1,EN2,EN3)                                             000050
C*                                                                      000060
C****************************************************************       000070
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)                               000080
       DIMENSION FMU(NSN),FN(NPX,NGR,NSN),S0(NIXA,NGR,NSN),             000090
     1           S1(NIXA,NGR,NSN),S2(NIXA,NGR,NSN),NC(NIXA),X(NPX)      000100
     2           ,ST(NGR,NMA),XIMP(IMP),FIMP(IMP,NGR,NSN),FPN(NIXA,NGR, 000110
     1           NSN),FPNX(NIXA,NGR,NSN),WMU(NSN),FU(4),WU(4)           000140
       EN0=1.D0                                                         000150
       EN1=1.D0                                                         000160
       EN2=1.D0                                                         000170
       EN3=1.D0                                                         000180
       FU(1)=.861136311594053D0                                         000190
       FU(2)=.339981043584856D0                                         000200
       FU(3)=-.339981043584856D0                                        000210
       FU(4)=-.861136311594053D0                                        000220
       WU(1)=0.347854845137454D0*.5D0                                   000230
       WU(2)=0.652145154862546D0*.5D0                                   000240
       WU(3)=0.652145154862546D0*.5D0                                   000250
       WU(4)=0.347854845137454D0*.5D0                                   000260
       CALL MOMEN(Q0,Q1,Q2,Q3)                                          000270
       DX=X(2)-X(1)                                                     000280
       EI=ST(1,1)*DX/FMU(1)                                             000290
       R0=0.D0                                                          000300
       R1=0.D0                                                          000310
       R2=0.D0                                                          000320
       R3=0.D0                                                          000330
       IF(NOPT.EQ.2) GOTO 3                                             000340
       IF(NOPT.EQ.3)GOTO 4                                              000350
       DO 1 I=NIXA/2+1,NIXA                                             000360
       DX=X(I+1)-X(I)                                                   000370
       P0=FN(I+1,1,1)                                                   000380
       P1=0.D0                                                          000390
       P2=0.D0                                                          000400
       P3=0.D0                                                          000410
       R0=R0+DX*P0                                                      000420
       R1=R1+DX*DX*P1/(3.D0)+DX*(X(I+1)+X(I)-1.D0)                      000430
     1 *P0                                                              000440
       R2=R2+DX*DX*DX*P2*                                               000450
     1 1.D0/5.D0+3.D0*DX*DX*(X(I+1)+X(I)-1.D0)*P1/3.D0                  000460
     2 +DX*(0.5D0*DX*DX+1.5D0*(X(I+1)+X(I))*(X(I+1)+X(I))-              000470
     3 3.D0*(X(I+1)+X(I))+1.D0)*P0                                      000480
       DX2=DX*DX                                                        000490
       DX3=DX*DX2                                                       000500
       DXP=X(I+1)+X(I)                                                  000510
       DXP2=DXP*DXP                                                     000520
       DXP3=DXP*DXP2                                                    000530
       DXPM1=DXP-1.D0                                                   000540
       R3=R3+0.5D0*DX*(2.D0*DX3*P3/7.D0+2.D0*DX2*DXPM1*P2+(DX3          000550
     1 +5.D0*DX*DXP2-10.D0*DXP*DX+4.D0*DX)*P1+(5.D0*DX2*                000560
     2 DXPM1+5.D0*DXP3-15.D0*DXP2+12.D0*DXP-2.D0)*P0)                   000570
   1   CONTINUE                                                         000580
       R1=R1*3.D0                                                       000590
       R2=R2*5.D0                                                       000600
       R3=R3*7.D0                                                       000610
       EN0=DABS(R0-Q0)                                                  000620
       EN1=DABS(R1-Q1)                                                  000630
       EN2=DABS(R2-Q2)                                                  000640
       EN3=DABS(R3-Q3)                                                  000650
       WRITE(6,*) R0,R1,R2,R3                                           000660
       WRITE(6,*) 'ERROR MOMENTO CERO=',EN0                             000670
       WRITE(6,*) 'ERROR MOMENTO UNO=',EN1                              000680
       WRITE(6,*) 'ERROR MOMENTO DOS=',EN2                              000690
       WRITE(6,*) 'ERROR MOMENTO TRES=',EN3                             000700
       GOTO 8                                                           000710
  3    CONTINUE                                                         000720
       R1=0.D0                                                          000730
       DO 2 I=NIXA/2+1,NIXA                                             000740
       DX=X(I+1)-X(I)                                                   000750
       P1=FN(I+1,1,1)-FPN(I,1,1)                                        000760
       P2=0.D0                                                          000770
       P3=0.D0                                                          000780
       R0=R0+DX*FPN(I,1,1)                                              000790
       R1=R1+DX*DX*P1/3.D0+DX*(X(I+1)+X(I)-1.                           000800
     1 D0)*FPN(I,1,1)                                                   000810
       R2=R2+DX*DX*DX*P2*                                               000820
     1 1.D0/5.D0+3.D0*DX*DX*(X(I+1)+X(I)-1.D0)*                         000830
     2 P1/3.D0+DX*(0.5D0*DX*DX+1.5D0*(X(I+1)+X(I))*(X(I+1)+X(I))-       000840
     3 3.D0*(X(I+1)+X(I))+1.D0)*FPN(I,1,1)                              000850
       DX2=DX*DX                                                        000860
       DX3=DX*DX2                                                       000870
       DXP=X(I+1)+X(I)                                                  000880
       DXP2=DXP*DXP                                                     000890
       DXP3=DXP*DXP2                                                    000900
       DXPM1=DXP-1.D0                                                   000910
       R3=R3+0.5D0*DX*(2.D0*DX3*P3/7.D0+2.D0*DX2*DXPM1*P2+(DX3          000920
     1 +5.D0*DX*DXP2-10.D0*DXP*DX+4.D0*DX)*P1+(5.D0*DX2*                000930
     2 DXPM1+5.D0*DXP3-15.D0*DXP2+12.D0*DXP-2.D0)*FPN(I,1,1))           000940
    2  CONTINUE                                                         000950
       R1=R1*3.D0                                                       000960
       R2=R2*5.D0                                                       000970
       R3=R3*7.D0                                                       000980
       WRITE(6,*) R0,R1,R2,R3                                           000990
       EN0=DABS(R0-Q0)                                                  001000
       EN1=DABS(R1-Q1)                                                  001010
       EN2=DABS(R2-Q2)                                                  001020
       EN3=DABS(R3-Q3)                                                  001030
       WRITE(6,*) 'ERROR MOMENTO CERO=',EN0                             001040
       WRITE(6,*) 'ERROR MOMENTO UNO=',EN1                              001050
       WRITE(6,*) 'ERROR MOMENTO DOS=',EN2                              001060
       WRITE(6,*) 'ERROR MOMENTO TRES=',EN3                             001070
       GOTO 8                                                           001080
  4    CONTINUE                                                         001090
       R2=0.D0                                                          001100
       R3=0.D0                                                          001110
       DO 7 I=NIXA/2+1,NIXA                                             001120
       DX=X(I+1)-X(I)                                                   001130
       P2=FN(I+1,1,1)-FPN(I,1,1)-FPNX(I,1,1)*3.0D0                      001140
       P3=0.D0                                                          001150
       R0=R0+DX*FPN(I,1,1)                                              001160
       R1=R1+DX*DX*FPNX(I,1,1)+DX*(X(I+1)+X(I)-1.                       001170
     1 D0)*FPN(I,1,1)                                                   001180
       R2=R2+DX*DX*DX*P2*                                               001190
     1 1.D0/5.D0+3.D0*DX*DX*(X(I+1)+X(I)-1.D0)*FPNX(I,1,1)              001200
     2 +DX*(0.5D0*DX*DX+1.5D0*(X(I+1)+X(I))*(X(I+1)+X(I))-              001210
     3 3.D0*(X(I+1)+X(I))+1.D0)*FPN(I,1,1)                              001220
       DX2=DX*DX                                                        001230
       DX3=DX*DX2                                                       001240
       DXP=X(I+1)+X(I)                                                  001250
       DXP2=DXP*DXP                                                     001260
       DXP3=DXP*DXP2                                                    001270
       DXPM1=DXP-1.D0                                                   001280
       R3=R3+0.5D0*DX*(2.D0*DX3*P3/7.D0+2.D0*DX2*DXPM1*P2+(DX3          001290
     1 +5.D0*DX*DXP2-10.D0*DX*DXP+4.D0*DX)*FPNX(I,1,1)*3.D0+(5.D0*DX2*  001300
     2 DXPM1+5.D0*DXP3-15.D0*DXP2+12.D0*DXP-2.D0)*FPN(I,1,1))           001310
  7    CONTINUE                                                         001320
       R1=R1*3.D0                                                       001330
       R2=R2*5.D0                                                       001340
       R3=R3*7.D0                                                       001350
       WRITE(6,*) R0,R1,R2,R3                                           001360
       EN0=DABS(R0-Q0)                                                  001370
       EN1=DABS(R1-Q1)                                                  001380
       EN2=DABS(R2-Q2)                                                  001390
       EN3=DABS(R3-Q3)                                                  001400
       WRITE(6,*) 'ERROR MOMENTO CERO=',EN0                             001410
       WRITE(6,*) 'ERROR MOMENTO UNO=',EN1                              001420
       WRITE(6,*) 'ERROR MOMENTO DOS=',EN2                              001430
       WRITE(6,*) 'ERROR MOMENTO TRES=',EN3                             001440
  8    XMAX=X(NPX)                                                      001450
       XMIN=X(1)                                                        001460
       N=1                                                              001470
       DXIMP=(XMAX-XMIN)/FLOAT(IMP-1)                                   001480
       DO 10 I=1,IMP                                                    001490
       XIMP(I)=XMIN+DXIMP*FLOAT(I-1)                                    001500
       IF(X(N+1).LT.XIMP(I))N=N+1                                       001510
       DO 10 J=1,NGR                                                    001520
       DO 10 K=1,NSN                                                    001530
       FIMP(I,J,K)=FPQ(NMA,NIXA,NPX,NGR,NSN,RLM,NOPT,X,NC               001540
     1 ,FMU,FN,S0,S1,S2,ST,N,J,K,XIMP(I),FPN,FPNX)                      001550
   10  CONTINUE                                                         001570
       ERIN=0.D0                                                        001580
       ERDN=0.D0                                                        001590
       DO 20 I=1,NIXA                                                   001600
       ERI=DABS(FN(I+1,1,1)-FEXAC(X(I+1),1))                            001610
       IF(NOPT.EQ.1)ERD=DABS(FN(I+1,1,1)-FEXAC(X(I),1))
       IF(NOPT.EQ.2)ERD=DABS(-FN(I+1,1,1)+2.D0*FPN(I,1,1)-FEXAC         001620
     1 (X(I),1))                                                        001630
       IF(NOPT.EQ.3)ERD=DABS(FN(I+1,1,1)-6.D0*FPNX(I,1,1)-              001640
     1 FEXAC(X(I),1))                                                   001650
       IF(ERD.GT.ERDN)ERDN=ERD                                          001660
       IF(ERI.GT.ERIN) ERIN=ERI                                         001670
   20  CONTINUE                                                         001680
       WRITE(6,*) 'MAXIMA NORMA CARA DER.=',ERIN                        001690
       WRITE(6,*) 'MAXIMA NORMA CARA IZQ.=',ERDN                        001700
  11   EN4=0.D0                                                         001710
       DO 30 I=1,IMP                                                    001720
       FEX=FEXAC(XIMP(I),1)                                             001730
       EIJK=DABS(FIMP(I,1,1)-FEX)                                       001740
       IF(EIJK.GT.EN4) EN4=EIJK                                         001750
C      WRITE(6,*) XIMP(I),FIMP(I,1,2),FEX,EIJK                          001760
   30  CONTINUE                                                         001770
       WRITE(6,*) 'MAXIMA NORMA=',EN4                                   001780
       RETURN                                                           001790
       END                                                              001800
C***************************************************************        001810
C*                                                                      001820
       FUNCTION FEXAC(X,K)                                              001830
C*                                                                      001840
C*******************************************************************    001850
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)                               001860
       A=DSQRT(6.D0)                                                    001870
       B=3.D0+2.D0*DSQRT(2.D0)                                          001880
       C=DEXP(-A)+B*DEXP(A)                                             001890
       IF(K.EQ.1)FEXAC=-(DEXP(X*A)+B*DEXP(-X*A))/C+1.D0                 001900
       IF(K.EQ.2)FEXAC=-(B*DEXP(X*A)+DEXP(-X*A))/C+1.D0                 001910
       RETURN                                                           001920
       END                                                              001930
C***************************************************************        001940
       SUBROUTINE MOMEN(Q0,Q1,Q2,Q3)                                    001950
       IMPLICIT DOUBLE PRECISION(A-H,O-Z)                               001960
       A1=DSQRT(6.D0)                                                   001970
       EP=DEXP(A1)                                                      001980
       EM=DEXP(-A1)                                                     001990
       A=-1.D0/(EM+(3.D0+2.D0*DSQRT(2.D0))*EP)                          002000
       B=(3.D0+2.D0*DSQRT(2.D0))*A                                      002010
       X1=1.D0-1.D0/A1                                                  002020
       X2=1.D0+1.D0/A1                                                  002030
       T1=6.D0*A*(1.D0-2.D0*X1/A1)*EP-6.D0*B*(1.D0+2.D0*X2/A1)*EM       002040
       T1=T1/A1                                                         002050
       T2=-6.D0*A*X1*EP+6.D0*B*X2*EM                                    002060
       T2=T2/A1                                                         002070
       T3=A*(-1.D0+EP)-B*(-1.D0+EM)-12.D0*(A-B)/(A1*A1)-6.D0*(A+B)      002080
     1 /A1                                                              002090
       T3=T3/A1                                                         002100
       Q2=T1+T2+T3                                                      002110
       Q2=Q2*5.D0                                                       002120
       X3=1.D0/A1-2.D0*X1/(A1*A1)                                       002130
       X4=-1.D0/A1-2.D0*X2/(A1*A1)                                      002140
       P1=20.D0*A*(1.D0/A1-3.D0*X3/A1)*EP+20.D0*B*(-1.D0/A1+3.D0*X4     002150
     1 /A1)*EM                                                          002160
       P2=-30.D0*A*X3*EP-30.D0*B*X4*EM                                  002170
       P3=12.D0*A*X1*EP/A1-12.D0*B*X2*EM/A1-A*EP/A1+B*EM/A1             002180
       P4=120*(A+B)/(A1*A1*A1*A1)+60.D0*(A-B)/(A1*A1*A1)+12.D0*(A+B)    002190
     1 /(A1*A1)+(A-B)/A1                                                002200
       Q3=P1+P2+P3+P4                                                   002210
       Q3=Q3*7.D0                                                       002220
       Q0=A*(EP-1.D0)-B*(EM-1.D0)                                       002230
       Q0=Q0/A1+1.D0                                                    002240
       Q1=2.D0*A*X1*EP-2.D0*B*X2*EM-A*EP+B*EM                           002250
       Q1=Q1/A1+2.D0*(A+B)/(A1*A1)+(A-B)/A1                             002260
       Q1=Q1*3.D0                                                       002270
       WRITE(6,*) Q0,Q1,Q2,Q3                                           002280
       RETURN                                                           002290
       END                                                              002300
